create database pytheas;
use pytheas;

#
# MySQL PYTHEAS table creation script
#

CREATE TABLE resource
(
  id        int  NOT NULL auto_increment,
  ctime	    timestamp,
  mtime     timestamp,
  uri varchar(255),
  blob_val  text,

  PRIMARY KEY (id)
);

create unique index resource_pk
   on resource ( id );

CREATE TABLE topicmap
(
  id        int  NOT NULL auto_increment,
  ctime	    timestamp,
  mtime     timestamp,
  topic_map_ident varchar(25),
  name_space varchar(255),
  xlink varchar(255),
  xml_base varchar(255),

  PRIMARY KEY (id)
);

create unique index topicmap_pk
   on topicmap ( id );

CREATE TABLE topic
(
  id        int  NOT NULL auto_increment,
  topicmap_id        int  NOT NULL,
  ctime	    timestamp,
  mtime     timestamp,
  resource_blob  text,
  subject_ind_ref int,
  resource_ref int,
  resource_data varchar(255),
  topic_ident varchar(25),
  is_instance int,
  is_published int,
  is_member int,
  is_occurance int,
  is_role int,

  PRIMARY KEY (id)
);

create unique index topic_pk
   on topic ( id );

CREATE TABLE ident
(
  id        int  NOT NULL auto_increment,
  topic_id        int  NOT NULL,
  ctime	    timestamp,
  mtime     timestamp,
  topic_ident varchar(25),

  PRIMARY KEY (id)
);

create unique index ident_pk
   on ident ( id );

CREATE TABLE base_name
(
  id        int  NOT NULL auto_increment,
  topic_id        int  NOT NULL,
  this_topic_id        int,
  ctime	    timestamp,
  mtime     timestamp,
  base_name_string varchar(255),

  PRIMARY KEY (id)
);

create unique index base_name_pk
   on base_name ( id );

CREATE TABLE variant
(
  id        int  NOT NULL auto_increment,
  base_name_id        int  NOT NULL,
  ctime	    timestamp,
  mtime     timestamp,
  variant_name varchar(255),

  PRIMARY KEY (id)
);

create unique index variant_pk
   on variant ( id );

CREATE TABLE parm
(
  id        int  NOT NULL auto_increment,
  variant_id        int  NOT NULL,
  ctime	    timestamp,
  mtime     timestamp,
  topic_ident varchar(25),
  topic_id int NOT NULL,

  PRIMARY KEY (id)
);

create unique index parm_pk
   on parm ( id );

CREATE TABLE occurance
(
  id        int  NOT NULL auto_increment,
  topic_id        int  NOT NULL,
  ctime	    timestamp,
  mtime     timestamp,
  topic_ident varchar(25),

  PRIMARY KEY (id)
);

create unique index occurance_pk
   on occurance ( id );

CREATE TABLE associations
(
  association_id        int  NOT NULL auto_increment,
  topic_id        int  NOT NULL,
  ctime	    timestamp,
  mtime     timestamp,
  topic_ident varchar(255),

  PRIMARY KEY (association_id)
);

create unique index associations_pk
   on associations ( association_id );

CREATE TABLE member
(
  id        int  NOT NULL auto_increment,
  topic_id        int  NOT NULL,
  this_role        int,
  association_id        int  NOT NULL,
  ctime	    timestamp,
  mtime     timestamp,
  topic_ident varchar(25),
  role_ident varchar(25),

  PRIMARY KEY (id)
);

create unique index member_pk
   on member ( id );

CREATE TABLE topic_associations_topic (
  many_id        int  NOT NULL auto_increment,
  id   int not null,
  association_id	int not null,

  PRIMARY KEY (many_id)
);

create index topic_associations_topic_t_pk on topic_associations_topic ( id );

create index topic_associations_topic_a_pk on topic_associations_topic ( association_id );

CREATE TABLE scope
(
  id        int  NOT NULL auto_increment,
  topic_id        int  NOT NULL,
  ctime	    timestamp,
  mtime     timestamp,
  topic_ident varchar(25),

  PRIMARY KEY (id)
);

create unique index scope_pk
   on scope ( id );

CREATE TABLE topic_scopes_topic (
  many_id        int  NOT NULL auto_increment,
  id   int not null,
  scope_id	int not null,
  topic_ident varchar(25),

  PRIMARY KEY (many_id)
);

create index topic_scopes_topic_s_pk on topic_scopes_topic ( id );

create index topic_scopes_topic_t_pk on topic_scopes_topic ( scope_id );

CREATE TABLE occurance_scopes_occurance (
  many_id        int  NOT NULL auto_increment,
  id   int not null,
  scope_id	int not null,
  topic_ident varchar(25),

  PRIMARY KEY (many_id)
);

create index occurance_scopes_occurance_s_pk on occurance_scopes_occurance ( id );

create index occurance_scopes_occurance_o_pk on occurance_scopes_occurance ( scope_id );

CREATE TABLE variant_scopes_variant (
  many_id        int  NOT NULL auto_increment,
  id   int not null,
  scope_id	int not null,

  PRIMARY KEY (many_id)
);

create index variant_scopes_variant_s_pk on variant_scopes_variant ( id );

create index variant_scopes_variant_v_pk on variant_scopes_variant ( scope_id );

CREATE TABLE parm_scopes_parm (
  many_id        int  NOT NULL auto_increment,
  id   int not null,
  scope_id	int not null,

  PRIMARY KEY (many_id)
);

create index parm_scopes_parm_s_pk on parm_scopes_parm ( id );

create index parm_scopes_parm_p_pk on parm_scopes_parm ( scope_id );

CREATE TABLE marc
(
  id        int  NOT NULL auto_increment,
  ctime	    timestamp,
  mtime     timestamp,

  PRIMARY KEY (id)
);

create unique index marc_pk
   on marc ( id );

CREATE TABLE rdf
(
  id        int  NOT NULL auto_increment,
  ctime	    timestamp,
  mtime     timestamp,
  resource_id        int  NOT NULL,
  is_type   int,
  is_bag    int,
  is_ord    int,
  seq_no    int,
  bag_no    int,
  subject   int,
  predicate varchar(100),
  value     varchar(255),
  name_space varchar(255),
  name_code varchar(25),
  normal_entry varchar(255),
  INDEX predname (predicate(30),value(100)),
  INDEX namepred (value(100),predicate(30)),
  
  PRIMARY KEY (id)
);

create unique index rdf_pk
   on rdf ( id );

CREATE TABLE webont
(
  webont_id        int  NOT NULL auto_increment,
  ctime	    timestamp,
  mtime     timestamp,
  name_space varchar(255),
  class_name varchar(255),
  class_type varchar(25),
  label varchar(255),
  comment varchar(255),
  ont_code varchar(25),
  ont_value varchar(255),
  resource        int,

  PRIMARY KEY (webont_id)
);

create unique index webont_pk
   on webont ( webont_id );

CREATE TABLE marc_vartag 
(
  id     int  NOT NULL auto_increment,
  ctime  timestamp,
  mtime  timestamp,
  marc_id  int, 
  tag_name varchar(3) not null,
  ind1     varchar(1) not null,
  ind2     varchar(1) not null,
  control_flag 	int,
  control_uri 	varchar(255),

  PRIMARY KEY (id)
);

create unique index marc_vartag_pk
   on marc_vartag ( id );

CREATE TABLE marc_vartag_subf (
  id     int  NOT NULL auto_increment,
  ctime  timestamp,
  mtime  timestamp,
  marcvartag_id	int not null, 
  seq_no  	int not null, 
  subf_code	varchar(1) not null,
  subf_value	varchar (255) not null,
  INDEX subf (subf_value(150)),

  PRIMARY KEY (id)
);

create unique index marc_vartag_subf_pk
   on marc_vartag_subf ( id );

CREATE TABLE bib (
  id	int  NOT NULL auto_increment,
  resource_id        int  NOT NULL,
  ctime  timestamp,
  mtime  timestamp,
  orig_no		varchar(100),
  record_status 	varchar(1),
  record_type 		varchar(1),
  encoding_level 	varchar(1),
  des_cat_form 		varchar(1),
  type_date 		varchar(1),
  beg_pub_date 		varchar(4),
  end_pub_date 		varchar(4),
  pub_place 		varchar(3),
  ill_code 		varchar(4),
  target_aud 		varchar(1),
  item_form		varchar(1),
  cont_nature		varchar(4),
  gov_code		varchar(1),
  conf_code		varchar(1),
  festschrift		varchar(1),
  own_index		varchar(1),
  fiction		varchar(1),
  biography		varchar(1),
  lang_code		varchar(3),

  PRIMARY KEY (id)
);

create unique index bib_pk
   on bib ( id );

CREATE TABLE holding (
  id	int  NOT NULL auto_increment,
  bib_id int  NOT NULL,
  ctime  timestamp,
  mtime  timestamp,
  record_status 	varchar(1),

  PRIMARY KEY (id)
);

create unique index holding_pk
   on holding ( id );

CREATE TABLE authority (
  id	int  NOT NULL auto_increment,
  ctime  timestamp,
  mtime  timestamp,
  record_status 	varchar(1),

  PRIMARY KEY (id)
);

create unique index authority_pk
   on authority ( id );

CREATE TABLE names (
  names_id	int  NOT NULL auto_increment,
  authority_id int,
  ctime  timestamp,
  mtime  timestamp,
  heading_type 	varchar(2),
  uri 		varchar(255),
  name 		varchar(255) not null,
  is_crossref int,
  is_established int,

  PRIMARY KEY (names_id)
);

create unique index names_pk
   on names ( names_id );


CREATE TABLE subjects (
  subjects_id	int  NOT NULL auto_increment,
  authority_id int,
  ctime  timestamp,
  mtime  timestamp,
  heading_type 	varchar(2),
  uri 		varchar(255),
  subject 	varchar(255) not null,
  is_crossref int,
  is_established int,

  PRIMARY KEY (subjects_id)
);

create unique index subjects_pk
   on subjects ( subjects_id );

CREATE TABLE subdivs (
  subdivs_id	int  NOT NULL auto_increment,
  authority_id int,
  ctime  timestamp,
  mtime  timestamp,
  heading_type 	varchar(2),
  uri 		varchar(255),
  subdiv 	varchar(255) not null,

  PRIMARY KEY (subdivs_id)
);

create unique index subdivs_pk
   on subdivs ( subdivs_id );

CREATE TABLE resource_names_resource (
  many_id        int  NOT NULL auto_increment,
  id   int not null,
  names_id	int not null,

  PRIMARY KEY (many_id)
);

create index resource_names_resource_r_pk on resource_names_resource ( id );

create index resource_names_resource_n_pk on resource_names_resource ( names_id );

CREATE TABLE resource_subjects_resource (
  many_id        int  NOT NULL auto_increment,
  id   int not null,
  subjects_id	int not null,

  PRIMARY KEY (many_id)
);

create index resource_subjects_resource_r_pk on resource_subjects_resource ( id );

create index resource_subjects_resource_s_pk on resource_subjects_resource ( subjects_id );

CREATE TABLE subjects_subdivs_subjects (
  many_id        int  NOT NULL auto_increment,
  subdivs_id   int not null,
  subjects_id	int not null,

  PRIMARY KEY (many_id)
);

create index subjects_subdivs_subjects_s_pk on subjects_subdivs_subjects ( subdivs_id );

create index subjects_subdivs_subjects_su_pk on subjects_subdivs_subjects ( subjects_id );

create table resource_webonts_resource (
  many_id        int  NOT NULL auto_increment,
  id   int not null,
  webont_id	int not null,

  PRIMARY KEY (many_id)
);

create index resource_webonts_resource_r_pk on resource_webonts_resource ( id );

create index resource_webonts_resource_w_pk on resource_webonts_resource ( webont_id );
