/*
 * Test.java - a simple program for "quick and dirty" testing!
 *
 * 		Art Rhyno (arhyno@uwindsor.ca),June 1999
 *		(c) Copyright GNU General Public License (GPL)
 *
 * 		Revised:
 *			Aug. 2002 - cleaned up a bit
 *
 */

package pytheas;

//PYTHEAS
import pytheas.*;

//Google
import com.google.soap.search.*;

//Java utils
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;
import java.util.Enumeration;
import java.io.*;
import java.io.PrintWriter;
import java.io.InputStreamReader;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.DriverManager;

//Castor
import org.xml.sax.ContentHandler;
import org.exolab.castor.jdo.JDO;
import org.exolab.castor.jdo.Database;
import org.exolab.castor.jdo.OQLQuery;
import org.exolab.castor.jdo.QueryResults;
import org.exolab.castor.util.Logger;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;
import org.exolab.castor.mapping.Mapping;
import org.apache.xml.serialize.*;


public class Test
{

	// Use for multiple loading and mapping files for testing

	public static final String DatabaseFile =	"database.xml";
	public static final String DatabaseFile2 =	"database2.xml";

	public static final String MappingFile = 	"mapping.xml";
	public static final String MappingFile2 = 	"display.xml";

    	private Mapping	_mapping, _mapping2;
    	private JDO	_jdo, _jdo2;


    	public static void main( String[] args )
    	{
        	PrintWriter   writer;
        	Test          test;
		FileOutputStream out = null;
        
		try {
			 out = new FileOutputStream("pytheas.log");
        	} catch ( IOException ex ) {
			System.out.println("logging problem");
		}//try

		// you can make the log go to stdout, e.g.:
        	//	writer = new Logger( System.out ).setPrefix( "pytheas" );

		writer = new Logger( out ).setPrefix( "pytheas" );
        	try {
            		test = new Test( writer );
            		test.run( writer );
        	} catch ( Exception except ) {
            		writer.println( except );
            		except.printStackTrace( writer );
        	}//try
	}//main

	public Test( PrintWriter writer )
        	throws Exception
    	{

		// Load the mapping files

		_mapping = new Mapping( getClass().getClassLoader() );
		_mapping.setLogWriter( writer );
		_mapping.loadMapping( getClass().getResource( MappingFile ) );

		_mapping2 = new Mapping( getClass().getClassLoader() );
        	_mapping2.setLogWriter( writer );
        	_mapping2.loadMapping( getClass().getResource( MappingFile2 ) );

        	_jdo = new JDO();
        	_jdo.setLogWriter( writer );
        	_jdo.setConfiguration( getClass().getResource( DatabaseFile ).toString() );
        	_jdo.setDatabaseName( "test" );

        	_jdo2 = new JDO();
        	_jdo2.setLogWriter( writer );
		_jdo2.setConfiguration( getClass().getResource( DatabaseFile2 ).toString() );
        	_jdo2.setDatabaseName( "test" );
    	}//Test

    	public void run( PrintWriter writer )
        	throws Exception
    	{
		// the usual suspects
        	Database	db, db2;
        	Marc		marc;
		Rdf		rdf;
        	Resource	resource;
        	MarcVarTag 	tag;
		MarcVarTagSubf 	subf;
		Names 		name;
        	Bib 		bib,fullbib;
		Names 		heading;

		// almost everything comes back to a query, so keep a bunch handy
        	OQLQuery	marcOql;
        	OQLQuery	rdfOql;
        	OQLQuery	resourceOql;
        	OQLQuery	bibOql;
        	OQLQuery	headingOql;
        	OQLQuery	subfOql;

		// query tools
		QueryResults	results;
        	Marshaller	marshaller;
		Enumeration enum;

		marshaller = new Marshaller( writer );
		marshaller.setMapping( _mapping2 );

		db2 = _jdo2.getDatabase();
		db2.begin();

		// I am not sure you would ever *want* to invoke validation at this level
		// but it's too buggy at this point anyway
		marshaller.setValidation(false);

        	db = _jdo.getDatabase();
        	db.begin();

		bibOql = db.getOQLQuery(
			"SELECT b from pytheas.Bib b where b.vartags.subfs.subfValue LIKE \"%war%\" LIMIT $1");
		bibOql.bind(5);

		// note - this is how to see what happens with a query
		writer.println("Query: " + 
			((org.exolab.castor.jdo.engine.OQLQueryImpl) bibOql).getSQL());

        	results = bibOql.execute(db.ReadOnly);
		if ( results instanceof Enumeration ) {
			enum = (Enumeration) results;
			while( enum.hasMoreElements() ) {
				bib = (Bib) enum.nextElement();

				// if using orig query - marshaller.marshal( (Bib) enum.nextElement() );
				// if using constructed object - marshaller.marshal( bib );
				// but here, try out second mapping
				marshaller.marshal( db2.load( Bib.class, new Integer(bib.getId()) ) );

				writer.println(bib.getId() + "\n\n");
			}//while
		}//if

		// clean up first
		db.commit();
		db.close();

		// clean up second
		db2.commit();
		db2.close();

		// if you are trying different databases, you may have to hunt down persistence errors,
		// this is how to catch the exceptions:

		/*
			db2.commit();

			try {
				db2.close();
			} catch (org.exolab.castor.jdo.PersistenceException ex) {
				writer.println("caught" + ex);
			}
		*/

		// this is how to invoke google

		/*
			try {
				GoogleSearch search = new GoogleSearch();
				search.setKey("whateveryoukeyis?");
				String suggest = search.doSpellingSuggestion("moouse");
				writer.println("google suggests: " + suggest);
			} catch (GoogleSearchFault ex) {
				System.out.println("caught" + ex);
			}
		*/
	}//run

}//Test
