/*
 * wdAdd - a quick program to load WordNet records.
 *
 * 		Art Rhyno (arhyno@uwindsor.ca), May 2002
 *		(c) Copyright GNU General Public License (GPL)
 *
 *		A more thought out program would use SAX for the
 *		the WordNet RDF dumps.
 */

package pytheas;

//PYTHEAS
import pytheas.*;

//Some standard Java packages
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.*;
import java.io.*;
import java.io.PrintWriter;
import java.io.InputStreamReader;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.DriverManager;

//SAX
import org.xml.sax.ContentHandler;
import org.apache.xml.serialize.*;

//Castor
import org.exolab.castor.jdo.JDO;
import org.exolab.castor.jdo.Database;
import org.exolab.castor.jdo.OQLQuery;
import org.exolab.castor.jdo.QueryResults;
import org.exolab.castor.util.Logger;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.mapping.Mapping;

public class wdAdd  {

    public static final String DatabaseFile = "database.xml";
    public static final String MappingFile = "mapping.xml";
    public static final String wdNameSpace = "http://www.semanticweb.org/library/wordnet/wordnet-20000620.rdfs";
    public static final String wdPrefix = "http://www.semanticweb.org/library/wordnet/";
    public static final String wdShort = "WN";

    private Mapping  _mapping;
    private JDO      _jdo;

    public static void main ( String args[] )
	throws ClassNotFoundException, IOException, InstantiationException, 
	IllegalAccessException, Exception
    {	    
	PrintWriter writer;
	FileOutputStream out = null;

	String input;
	String user = "";
	String password = "";
	String db = "";
	String host = "";
	String port = "";
	String match = "";

	String inputFile = "wdAdd.rdf";
	int maxFieldLen = 255;

	int j = args.length;
	for (int i=0;i<j;i++) {
		if (args[i].equals("-i") && i+1 < j)
			inputFile = args[++i];
		if (args[i].equals("-l") && i+1 < j)
			maxFieldLen = Integer.parseInt(args[++i]);
		if (args[i].equals("-m") && i+1 < j)
			match = args[++i];
		if (args[i].equals("-?")) {
			System.out.println("wdAdd [-i input file] " +
				"[-m match] [-l maximum length] [-?]");
			System.exit(1);
		}//if args
	}//for i

  	System.out.println("Starting, please wait...");
		
	// Clear log
	PrintWriter clearit = new PrintWriter(new FileOutputStream("wdAdd.log"));
	clearit.close();

	out = new FileOutputStream("pytheas.log");

	writer = new Logger(out).setPrefix("pytheas");

	wdAdd p = new wdAdd(writer,inputFile,maxFieldLen,match);
    }//main
    
    // wdAdd - process specified file into database
    public wdAdd(PrintWriter writer, String inputFile, int maxFieldLen, String match) 
	throws Exception
    {
	Database	db = null;
	String		input,theValue;
	int recordCnt = 0;

	Vector words = new Vector();
	Vector preds = new Vector();

        _mapping = new Mapping( getClass().getClassLoader() );
        _mapping.setLogWriter( writer );
        _mapping.loadMapping( getClass().getResource( MappingFile ) );

        _jdo = new JDO();
        _jdo.setLogWriter( writer );
        _jdo.setConfiguration( getClass().getResource( DatabaseFile ).toString() );
        _jdo.setDatabaseName( "test" );
	
	db = _jdo.getDatabase();

	Resource resource;

	DataInputStream D1 = new DataInputStream(new FileInputStream(inputFile));
	BufferedReader In1 = new BufferedReader(new InputStreamReader(D1)); 
    
	int i=0;
	int theIndex = 0;
	String thePredicate = "";
	String theSubject = "";
	Rdf rdf;

	boolean started = false;

	System.out.print("Processing...");
	while ((input = In1.readLine()) != null && recordCnt < 10000) {
		i = input.indexOf(match);
		if (i != -1) {
			if (input.indexOf("/") != -1)
				started = false;
			else {
				theSubject = input.substring(
					input.indexOf("about=") +7);
				theSubject = theSubject.substring(
					0,theSubject.indexOf("\""));
			}//if input
					
		}//if i

		if (i == -1 && started) {	
			i = 0;		
			if (input.indexOf("<") == -1)
				i++;
	    		String parm = "";
	    		String parmValue = "";
	    		StringTokenizer tokens = new StringTokenizer(input,"<>:=");
	    		while (tokens.hasMoreTokens() && i < 4) {
				parm = tokens.nextToken().trim();
				if (parm.startsWith("\"")) {
					parm = parm.substring(1);
					parm = parm.substring
						(0,parm.indexOf("\""));
					if (parm.length() > maxFieldLen)
						parm = parm.substring(0,
							maxFieldLen - 1);
				}//if
				if (i == 2)
					preds.addElement(parm);
				if (i == 3)
					words.addElement(parm);
				System.out.print(".");
				i++;
			}//while
		} else {
			if (i != -1)
				started = true;
			if (words.size() > 1) {
				db.begin();
				resource = new Resource();
				resource.setUri(wdPrefix + theSubject);
				db.create(resource);
				for (i=0;i<words.size() && i<preds.size();i++) {
					rdf = new Rdf();
					rdf.setResource(resource);
					rdf.setIsType(false);
					rdf.setPredicate((String)preds.elementAt(i));
					rdf.setNameSpace(wdNameSpace);
					rdf.setNameCode(wdShort);
					theValue = (String)words.elementAt(i);
					rdf.setValue(theValue);
					theValue = theValue.toUpperCase();
					theValue = theValue.trim();
					rdf.setNormalEntry(normalizeEntry(theValue));
					db.create(rdf);
					recordCnt++;
				}//for
				db.commit();
			}//if words
			words = new Vector();
			preds = new Vector();
		}//if i
	} // while input

	db.close();

	System.out.println ("\nProcessing Complete: " + recordCnt + 
		" record(s) added.");
    }//wdAdd
        
     
    public String normalizeEntry(String abnormalEntry) 
    {
	String theBadList = "[]';:<>/\"%_\\().,-+!@#$%^&*";
	String theCleanEntry = "";
	
	StringTokenizer tokens = new StringTokenizer(abnormalEntry,theBadList);

	while (tokens.hasMoreTokens()) 
		theCleanEntry = theCleanEntry + tokens.nextToken();

	return theCleanEntry;
    }//normalizeEntry
            
}//wdAdd
