/*
 * FaultMessage.java - handle messages when things go wrong.
 *
 * 		Art Rhyno (arhyno@uwindsor.ca), Sept. 2002
 *		(c) Copyright GNU General Public License (GPL)
 *
 * 		Revised:
 *
 */

package pyserv;

import java.util.Iterator;

//SOAP
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPElement;	
import javax.xml.soap.SOAPFault;
import javax.xml.soap.Name;
import javax.xml.soap.Detail;
import javax.xml.soap.DetailEntry;

public class FaultMessage {

	SOAPMessage message;
	SOAPFault fault;

	public FaultMessage(MessageFactory messageFactory)
		throws SOAPException {

		message = messageFactory.createMessage();

		SOAPPart soapPart;
		SOAPEnvelope soapEnvelope;
		SOAPBody soapBody;

		soapPart = message.getSOAPPart();
		soapEnvelope = soapPart.getEnvelope();
		soapBody = soapEnvelope.getBody();

		fault = soapBody.addFault();
	}//FaultMessage

	public void setFaultCode(String faultCode)
			throws SOAPException 
	{
		fault.setFaultCode(faultCode);
	}//setFaultCode

	public void setFaultString(String faultString)
		throws SOAPException 
	{

		fault.setFaultString(faultString);
	}//setFaultString

	public void setFaultDetail(String faultDetail)
		throws SOAPException 
	{
		SOAPEnvelope soapEnvelope =
			message.getSOAPPart().getEnvelope();

		Detail detail = fault.addDetail();

		Name name = soapEnvelope.createName(
			"FaultDetail",
			"pyns",
			"soap-example");

		DetailEntry entry =
			detail.addDetailEntry(name);

		entry.addTextNode(faultDetail);
	}//setFaultDetail

	public SOAPMessage getMessage() 
	{
		return message;
	}//getMessage
}//FaultMessage
