/*
 * Pyxml - a servlet to work with PYTHEAS database structure.
 *
 * 		Art Rhyno (arhyno@uwindsor.ca), Mar. 2002
 *		(c) Copyright GNU General Public License (GPL)
 *
 * 		Revised:
 *			Apr. 2002 - changed heading/authorities structure
 *
 */

package pyserv;

//PYTHEAS Classes
import pytheas.*;

//Google API
import com.google.soap.search.*;

//Standard Java
import java.net.*;
import java.io.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Enumeration;

//Servlet
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

//Apache
import org.apache.xml.serialize.*;

//SAX
import org.xml.sax.InputSource;
import org.xml.sax.helpers.AttributeListImpl;

//Castor
import org.exolab.castor.jdo.Database;
import org.exolab.castor.jdo.OQLQuery;
import org.exolab.castor.jdo.QueryResults;
import org.exolab.castor.jdo.JDO;
import org.exolab.castor.util.Logger;
import org.exolab.castor.mapping.Mapping;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;


public class Pyxml
    extends XMLServlet
{

	// A few globals, some of this should go in web.xml!
   	public static final String briefMarcStyleSheet = "/pyserv/briefm.xsl";
   	public static int typeRDF = 0;
	public static int typeMARC = 1;
	public static int typeOBJ = 3;
	// Hacked for now, should use LIMIT (2,10) in OQL
	public static int displayLimit = 10;
	public static String tempOPT = " tempOpt ";
	String nameCode="nameCode";
	public static String wordNetQuery = "select r from pytheas.Rdf r where nameCode=\"WN\" and normalEntry=";

    	public void service( HttpServletRequest request, XMLServletResponse response)
        	throws IOException,ServletException
    	{

		// Initialize
		boolean useXMLout = false;
		boolean expandHTML = false;
		boolean getWords = false;
		boolean giveEditForm = false;
		boolean doBibUpdate = false;
		boolean doBriefBibMapping = false;
		boolean doFullBibMapping = false;
		boolean useWordNet = false;
		boolean useGoogle = false;
		boolean loadDirect = false;

		int theID = 0;

		String test="";
		String objQuery = "";
		int i;
		int theType = 0;
		String focus = "";

		String rdfHTML = getPyUrl() + "rdf.html";
		String marcHTML = getPyUrl() + "marc.html";

		Vector termSugs = new Vector();

		if (request.getParameter("boolean") != null && request.getParameter("boolean").
			length() != 0) {
			expandHTML = true;
			test = request.getParameter("boolean");

			for (i=0;  request.getParameter("boolean_" +i) != null;	
				test = request.getParameter("boolean_" +i),i++);
			if (test.length() == 0)
				expandHTML = false;
			else 
				focus = "_" + i;
		}//if

		if (request.getParameter("loadDirect") != null) {
			loadDirect = true;
			theID = Integer.parseInt(request.
					getParameter("loadDirect"));
			theType = typeMARC;
		}//if

		if (request.getParameter("showXML") != null || loadDirect) 
			useXMLout = true;

		if (request.getParameter("useBrief") != null) 
			doBriefBibMapping = true;

		if (request.getParameter("useFull") != null || loadDirect) 
			doFullBibMapping = true;

		if (request.getParameter("wordNet") != null) 
			useWordNet = true;

		if (request.getParameter("googleAPI") != null) 
			useGoogle = true;

		test = request.getParameter("requestType");
		if (test != null) {
			// By default is type Rdf
			if (test.indexOf("marc") != -1)
				theType=typeMARC;
			else if (test.indexOf("obj") != -1)
				theType=typeOBJ;
			else
				useXMLout = true; // for now
		}//if		


          	// Set the output type and stylesheet if HTML
		// make XML for RDF until StyleSheet is ready
		if ((!useXMLout && theType != typeRDF) || expandHTML) {
			response.setContentType("text/html");
            		response.setOutputMethod( "html" );
			if (!expandHTML)
				response.setStylesheet( new InputSource(briefMarcStyleSheet));
		} else {
			response.setContentType( "text/xml" );
		}//if

		if (theType == typeOBJ || loadDirect)
			objQuery = request.getParameter("objQuery");
		else
			objQuery = new FormInfo(request,expandHTML,theType,response,
				focus,rdfHTML,marcHTML,useGoogle,termSugs,
				getGoogleKey()).toString();

		if (objQuery == null || objQuery.
			equals("select r from pytheas.Rdf r where nameCode =\"DC\" OR nameCode=\"BIB1\"")
		)
			objQuery = "";

		if (!expandHTML) {
			test = request.getParameter("updateRec");
			if ( test == null) {
				if (objQuery.length() > 0 || loadDirect) {
					System.out.println("-> " + objQuery);
					if (!runDisplayQuery(request,response,objQuery,theType,doBriefBibMapping,
						doFullBibMapping,useXMLout,loadDirect,theID,useGoogle,useWordNet,termSugs)
					)
						bailAndExit(response,"error");
				} else 
					bailAndExit(response,"No query!");
			} else {
				if (test.length() > 0) {
					if (null == new UpdateMarc(test))
						bailAndExit(response,"error");
					else
						bailAndExit(response,"updated");
				}//if test.length
			}//if test
		}//if
	}//service


	/*
		runDisplayQuery - this is where the query is actually passed to Castor
	*/
	public boolean runDisplayQuery(HttpServletRequest request, XMLServletResponse response, String objQuery,
		int theType, boolean doBriefBibMapping, boolean doFullBibMapping, boolean useXMLout,
		boolean loadDirect, int theID, boolean useGoogle, boolean useWordNet,
		Vector termSugs) throws ServletException
	{

		String googleSuggests="";
		String objQueryAddon = "";
		String wordNetTerm = "";

                try {
       	    		JDO		jdo = null;
			JDO		loadjdo = null;
			Database	db,directdb;
       	    		OQLQuery	oql = null;
			Object		result = null;
			Marshaller	marshal;
			Mapping 	mapping;
			QueryResults  	results;
			Bib 		bib;
			Rdf		rdf;


			//Mapping is done in Servlet ini, can just grab here
			if (doBriefBibMapping) {
				mapping = getBriefBibMapping();
				System.out.println("brief mapping");
			} else if (doFullBibMapping) {
				mapping = getFullBibMapping();
				System.out.println("full");
			} else {
				if (theType == typeRDF)
					mapping = getBriefRdfMapping();
				else
					mapping = getMapping();                      
				System.out.println("default");
			}//if

			/*
            			Load the mapping from the specified filed, and Open up the
       	    			database for read/write access
			*/
			if (useWordNet) {
				for (int i=0; i<termSugs.size();i++) {
					TermInfo tempInfo = (TermInfo) termSugs.elementAt(i);
					wordNetTerm = wordNetTerms(tempInfo.getVar(),tempInfo.getTerm(),objQuery);
					objQueryAddon = objQueryAddon + tackOnTerms(objQueryAddon,
						wordNetTerm);
					if (!tempInfo.getSuggestion().startsWith("problem:") &&
						tempInfo.getSuggestion().length() > 0) 
					{
						wordNetTerm = wordNetTerms(tempInfo.getVar(),
							tempInfo.getSuggestion(),objQuery);
						objQueryAddon = objQueryAddon + tackOnTerms(objQueryAddon,
							wordNetTerm);
					}//if
				}//for
			}//if

			if (objQuery.indexOf(nameCode) != -1)
				objQueryAddon = objQueryAddon + ")";

			System.out.println("addon : " + objQueryAddon);
	
			
			/*
			if (theType != typeMARC)
				jdo = getJDO();	
			else 
				jdo = getSearchJDO();
			*/

			jdo = getJDO();
        	    	db = jdo.getDatabase();
			db.begin();

			/* you could also use more than one jdo at a time
			loadjdo = getJDO();

        	    	directdb = loadjdo.getDatabase();
			directdb.begin();
			*/

			if (!loadDirect) {
				oql = db.getOQLQuery(objQuery + objQueryAddon);
				// oql = db.getOQLQuery("select r from pytheas.Rdf r where normalEntry like \" snake%\"");
				result = oql.execute(db.ReadOnly);
				System.out.println("done searching");
				//return true;
			}//if
	
			marshal = new Marshaller( response.getDocumentHandler() );
			marshal.setMapping(mapping);
			marshal.setValidation(false);
			// test
			//marshal.marshal( directdb.load( Bib.class, new Integer(2) ) );

			response.getDocumentHandler().startDocument();

			if (!loadDirect) {
				if (useXMLout) {
					if (useGoogle) {
						for (int i=0; i<termSugs.size();i++) {
							TermInfo tempInfo = (TermInfo) termSugs.elementAt(i);
							if (tempInfo.isGoogle)
								googleSuggests = " " + googleSuggests +
									tempInfo.getSuggestion() ;
						}
					}//if
					response.getDocumentHandler().startElement( "!-- " + objQuery + 
						objQueryAddon + "--", 
						new AttributeListImpl() );
					if (googleSuggests.length() > 0)
						response.getDocumentHandler().startElement( "!-- Google suggests:" +
							googleSuggests + "--", 
							new AttributeListImpl() );
					System.out.println("element added");
				}//if
				response.getDocumentHandler().startElement( "records", new AttributeListImpl() );
		
				// When lazy loading is set up, will need to go this route
				if ( result instanceof Enumeration ) {
					Enumeration enum;
					enum = (Enumeration) result;
					int theLimit = 0;

					while( enum.hasMoreElements() && theLimit++ < displayLimit) {
						if (theType == typeMARC) {
							marshal.marshal( (Bib) enum.nextElement() );
							/*
							bib = (Bib) enum.nextElement();
							System.out.println("about to marshall: " + bib.getId());
							marshal.marshal( directdb.load( Bib.class, new Integer(bib.getId())));
							*/
						} else
							marshal.marshal( (Rdf) enum.nextElement() );
					}
        		    	} else 
                			marshal.marshal( result );

				//close query
				oql.close();

				response.getDocumentHandler().endElement( "records" );
			} else {
				marshal.marshal(db.load(Bib.class, new Integer(theID)));
			}//if
			response.getDocumentHandler().endDocument();

			db.commit();
			db.close();
			return true;

		} catch (IllegalArgumentException iexcept) {
			iexcept.printStackTrace();
			throw new ServletException( iexcept.toString());
		} catch ( Exception except ) {
			except.printStackTrace();
			throw new ServletException( except.toString());
		}

	} // runDisplayQuery


	public String tackOnTerms(String startWith, String theTerms)
	{
		String newString = "";
		if (theTerms.length() > 0) {
			if (startWith.length() > 0)
				newString = startWith + " ";
			newString = newString + theTerms;
		}//if
		
		return newString;
	}//tackOnTerms

	public String wordNetTerms(String theVar,String term, String objQuery) throws ServletException
	{

		String theTermsIfAny = "";
			System.out.println("trying!: " + wordNetQuery + "\"" + term + "\"");
		System.out.println("going for it?");
                try {
       	    		JDO		jdo;
			Database	db;
       	    		OQLQuery	oql,oql2;
			QueryResults  	results,results2;
			Rdf		rdf;
			Resource	resource;                   

            		// Load the mapping from the specified filed, and Open up the
       	    		// database for read/write access
	
			jdo = getSearchJDO();	
        	    	db = jdo.getDatabase();
			db.begin();

			System.out.println("trying!: " + wordNetQuery + "\"" + term + "\"");

	
			oql = db.getOQLQuery(wordNetQuery + "\"" + term + "\"");
			results = oql.execute(db.ReadOnly);
			System.out.println("past query");
			while (results.hasMore()) {
				rdf = (Rdf) results.next();
				System.out.println("getting direct");
				resource = (Resource) rdf.getResource();
				System.out.println("select r from pytheas.Rdf r where resource = $1");
				oql2 = db.getOQLQuery("select r from pytheas.Rdf r where resource = $1");
				oql2.bind(resource.getId());
				results2 = oql2.execute(db.ReadOnly);
				while (results2.hasMore()) {
					rdf = (Rdf) results2.next();
					String test = rdf.getNormalEntry();
					if (!term.equals(test) && objQuery.indexOf(test) == -1)
						theTermsIfAny = theTermsIfAny + " OR " + theVar + " LIKE \"% " + test + "%\"";
				}
				oql2.close();
			}
			oql.close();

			db.commit();
			db.close();

		} catch (IllegalArgumentException iexcept) {
			iexcept.printStackTrace();
			throw new ServletException( iexcept.toString());
		} catch ( Exception except ) {
			except.printStackTrace();
			throw new ServletException( except.toString());
		}

		return theTermsIfAny;

	} // runDisplayQuery

	public void bailAndExit(XMLServletResponse response, String message) 
		throws IOException
	{
		PrintWriter out = response.getWriter();
		out.println(message);
	}
}
