/*
 * UpdateMarc - an example of updating using XML.
 *
 *	Ideally, you would just unmarshall and do a direct update, BUT
 *	this raises some nasty issues with "many-to-many" relationships.
 *	Would also like to add some support for WebDAV.
 *
 * 		Art Rhyno (arhyno@uwindsor.ca), Mar. 2002
 *		(c) Copyright GNU General Public License (GPL)
 *
 * 		Revised:
 *			Sept. 2002 - revamped for lazy loading
 *
 */

package pyserv;

//PYTHEAS Classes
import pytheas.*;

//Standard Java
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Collection;
import java.util.Iterator;
import java.net.*;
import java.io.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Enumeration;

//Servlet
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

//Apache
import org.apache.xml.serialize.*;

//SAX
import org.xml.sax.InputSource;
import org.xml.sax.helpers.AttributeListImpl;

//Castor
import org.exolab.castor.jdo.Database;
import org.exolab.castor.jdo.OQLQuery;
import org.exolab.castor.jdo.QueryResults;
import org.exolab.castor.jdo.JDO;
import org.exolab.castor.util.Logger;
import org.exolab.castor.mapping.Mapping;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;


public class UpdateMarc
    extends XMLServlet
{
	public String updateStatus = null;	

	UpdateMarc(String modifiedRecord) throws ServletException
	{

		boolean updateFlag = false;
		Collection tagSet, subfSet;
		Iterator subfItor,tagItor;

                try {
			JDO		jdo;
			Mapping		mapping;
		        Database	db;
        		Marc       	marc;
			Rdf		rdf;
        		MarcVarTag 	tag;
			MarcVarTagSubf 	subf;
			Names		name;
			Subjects 	subject;
        		Bib 		bib,bibnew;
        		OQLQuery      	bibOql;
			QueryResults	results;
			Resource 	resource,resnew;

			mapping = getMapping();
			jdo = getJDO();
			db = jdo.getDatabase();
			db.begin();

			Reader reader = new StringReader(modifiedRecord);

			bibnew = (Bib)Unmarshaller.unmarshal(Bib.class, reader);
			resnew = bibnew.getResource();			
			bibnew.setCreationTime(bibnew.getCreationTime());
			bibnew.setModificationTime(new Date());

			tagSet = bibnew.getVartags();

			if (tagSet == null)
				tagSet = new ArrayList();

        		bibOql = db.getOQLQuery( "SELECT b FROM pytheas.Bib b WHERE id = $1" );
        		bibOql.bind( bibnew.getId() );
        		results = bibOql.execute();
        		while ( results.hasMore() ) {
           			bib = (Bib) results.next();
           			db.remove( bib );
			}//while
			db.commit();
			db.close();

			// Now that we have zapped the record, let's re-add it
			// to the database.
			db = jdo.getDatabase();
			db.begin();

			bib = new Bib();
			// Bibs are re-attached to Resource
			resource = (Resource) db.load( Resource.class,new Integer(resnew.getId()));
		
			bib.setResource( resource );
			bib.setCreationTime(bibnew.getCreationTime());
			bib.setModificationTime(new Date());
			bib.setTypeDate( bibnew.getTypeDate() );
			bib.setBegPubDate( bibnew.getBegPubDate() );
			bib.setEndPubDate( bibnew.getEndPubDate() );
			bib.setPubPlace( bibnew.getPubPlace() );
			bib.setIllCode( bibnew.getIllCode() );
			bib.setTargetAud( bibnew.getTargetAud() );
			bib.setItemForm( bibnew.getItemForm() );
			bib.setContNature( bibnew.getContNature() );
			bib.setGovCode( bibnew.getGovCode() );
			bib.setConfCode( bibnew.getConfCode() );
			bib.setFestschrift( bibnew.getFestschrift() );
			bib.setOwnIndex( bibnew.getOwnIndex() );
			bib.setFiction( bibnew.getFiction() );
			bib.setBiography( bibnew.getBiography() );
			bib.setLangCode( bibnew.getLangCode() );

			tagItor = tagSet.iterator();
			while (tagItor.hasNext()) {
				MarcVarTag theTag = (MarcVarTag) tagItor.next();
				tag = new MarcVarTag();
				tag.setTagName(theTag.getTagName());
				tag.setInd1(theTag.getInd1());
				tag.setInd2(theTag.getInd2());
				tag.setMarc(bib);
	
				subfSet = theTag.getSubfs();
				if (subfSet == null)
					subfSet = new ArrayList();
				subfItor = subfSet.iterator();
				while (subfItor.hasNext()) {
					MarcVarTagSubf theSubf = (MarcVarTagSubf) subfItor.next();
					subf = new MarcVarTagSubf();
					subf.setSeqNo(theSubf.getSeqNo());
					subf.setSubfCode(theSubf.getSubfCode());
					subf.setSubfValue(theSubf.getSubfValue());
					subf.setMarcvartag(tag);
				}
				tag.setSubfs(subfSet);
			}//while
		
			bib.setVartags(tagSet);
			db.create(bib);
			db.commit();
			db.close();	
			updateStatus = "updated";		
		} catch (IllegalArgumentException iexcept) {
			iexcept.printStackTrace();
			throw new ServletException( iexcept.toString());
		} catch ( Exception except ) {
			except.printStackTrace();
			throw new ServletException( except.toString());
		}//try

	}//UpdateMarc

    	public void service( HttpServletRequest request, XMLServletResponse response)
        	throws IOException,ServletException
    	{
		// need this to extend XMLServlet
	}//service

	public String toString()
	{
		return updateStatus;
	}//toString
}//UpdateMarc
