/*
 * XMLServlet.java - 	a servlet to work with PYTHEAS, this layout based 
 *			on Assaf Arkin's original example in the Castor CVS.
 *
 * 		Art Rhyno (arhyno@uwindsor.ca), Mar. 2002
 *		(c) Copyright GNU General Public License (GPL)
 *
 */

package pyserv;

//Standard Java
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Locale;

//SAX & Apache
import org.xml.sax.DocumentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;
import org.apache.xml.serialize.Serializer;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.SerializerFactory;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.Method;

//Servlet
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletContext;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

//Castor
import org.exolab.castor.jdo.JDO;
import org.exolab.castor.mapping.Mapping;
import com.kvisco.xsl.*;
import com.kvisco.xsl.util.*;
import com.kvisco.net.impl.URILocationImpl;

public abstract class XMLServlet
    extends HttpServlet
{

	public static final String DatabaseFile = "/database.xml";
	public static final String SearchingFile = "/search.xml";
	public static final String MappingFile =  "/mapping.xml";
	public static final String briefBibFile = "/bibbrief.xml";
	public static final String fullBibFile = "/bibfull.xml";
	public static final String briefRdfFile = "/rdfbrief.xml";

    	private ServletContext  _ctx;
    	private static XSLProcessor    _xslp;

    	private JDO	_jdo;
    	private JDO	_searchJdo;

    	private Mapping _mapping;
    	private Mapping _briefBibMapping;
    	private Mapping _fullBibMapping;
    	private Mapping _briefRdfMapping;

	private String xslPath;
	private String pyPath;
	private String pyUrl;
	private String googleKey;
    
    	public void init( ServletConfig config ) 
    	{

		_mapping = null;
		_jdo = null;
		_searchJdo = null;
		pyPath = null;
		pyUrl = null;

		try {

			// pull pieces out of web.xml file
			xslPath = config.getInitParameter("xslPath");
			pyPath = config.getInitParameter("serverPath");
			pyUrl = config.getInitParameter("serverUrl");
			googleKey = config.getInitParameter("googleKey");

			// do mapping - we do this here so it only has to happen once
			_mapping = new Mapping();
			_briefBibMapping = new Mapping();
			_fullBibMapping = new Mapping();
			_briefRdfMapping = new Mapping();
		
			System.out.println("loading full mapping");
			_mapping.loadMapping(xslPath + MappingFile);

			System.out.println("loading brief mapping");
			_briefBibMapping.loadMapping(xslPath + briefBibFile);

			System.out.println("loading bib mapping");
			_fullBibMapping.loadMapping(xslPath + fullBibFile);

			System.out.println("loading rdf mapping");
			_briefRdfMapping.loadMapping(xslPath + briefRdfFile);

	        	_jdo = new JDO();
        		_jdo.setDatabaseName( config.getInitParameter("databaseName") );
			_jdo.setConfiguration( xslPath + DatabaseFile );

	        	_searchJdo = new JDO();
        		_searchJdo.setDatabaseName( config.getInitParameter("databaseName") );
			_searchJdo.setConfiguration( xslPath + SearchingFile );
		} catch (java.io.IOException ie) {
			System.err.println("IOException: " + ie);
		} catch (org.exolab.castor.mapping.MappingException mape) {
			System.err.println("MappingException: " + mape);
		} //try

        	_ctx = config.getServletContext();
        	_xslp = new XSLProcessor();
	
    	}//init

	/*
		Simple get methods
	*/

	public JDO getJDO()
	{
		return _jdo;
	}//getJDO

	public JDO getSearchJDO()
	{
		return _searchJdo;
	}//getSearchJDO


	public Mapping getMapping()
	{
		return _mapping;
	}//getMapping

	public Mapping getBriefBibMapping()
	{
		return _briefBibMapping;
	}//getBriefBibMapping

	public Mapping getFullBibMapping()
	{
		return _fullBibMapping;
	}//getFullBibMapping

	public Mapping getBriefRdfMapping()
	{
		return _briefRdfMapping;
	}//getBriefRdfMapping

	public String getPyPath()
	{
		return pyPath;
	}//getPyPath

	public String getPyUrl()
	{
		return pyUrl;
	}//getPyUrl

	public String getGoogleKey()
	{
		return googleKey;
	}//getGoogleKey


	/*
		Standard servlet stuff
	*/
    	protected final void service( HttpServletRequest request, HttpServletResponse response)
        	throws ServletException, IOException
    	{
        	service( request, new XMLServletResponseImpl( _ctx, response ) );
    	}//service

    	protected abstract void service( HttpServletRequest request, XMLServletResponse response )
        	throws ServletException, IOException;


    	static final class XMLServletResponseImpl
        	implements XMLServletResponse
    	{

        	private DocumentHandler   _docHandler;

        	private XSLStylesheet   _stylesheet;

        	private OutputFormat      _format;

        	private final HttpServletResponse _response;
        
        	private final ServletContext      _ctx;

        	private final XSLReader           _xslReader;        

        	XMLServletResponseImpl( ServletContext ctx, HttpServletResponse response )
        	{
            		String param;

            		_ctx = ctx;
            		_response = response;
            		param = _ctx.getInitParameter( "xsl:stylesheet" );

            		if ( param != null )
                		setStylesheet( param );

            		param = _ctx.getInitParameter( "xsl:output-method" );

            		if ( param != null )
                		setOutputFormat( new OutputFormat( param, null, 
					"true".equals( 
					_ctx.getInitParameter( "xsl:output-indent" ) ) ) );
                
            		_xslReader = new XSLReader();
        	}//XMLServletResponseImpl

        	public DocumentHandler getDocumentHandler()
            		throws IOException, SAXException
        	{
            		if ( _docHandler != null )
                		return _docHandler;
                
            		Formatter _formatter = null;
            
            		if ( _format == null ) {
                
                		XMLSerializer ser;
                		_format = new OutputFormat( Method.XML, _response.getCharacterEncoding(), 
					false );
                		ser = new XMLSerializer( _format );
                		ser.setOutputCharStream( getWriter() );
                		_formatter = new FormatterAdapter(ser.asDocumentHandler());
            		} else {
                		Serializer ser;
                
                		_format.setEncoding( _response.getCharacterEncoding() );
                		ser = SerializerFactory.getSerializerFactory( _format.getMethod() )
					.makeSerializer( _format );
                		ser.setOutputCharStream( getWriter() );
                		_formatter = new FormatterAdapter(ser.asDocumentHandler());
            		}//if

            		_response.setContentType( "text/" + _format.getMethod() );

            		if ( _stylesheet != null ) {
                		SAXInput saxInput = new SAXInput();
                		saxInput.setProcessor(_xslp);
                		saxInput.setOutputHandler(_formatter);
                		saxInput.setStylesheet(_stylesheet);
                		_docHandler = saxInput;
            		} else 
				_docHandler = _formatter;
            
            		return _docHandler;
        	}//getDocumentHandler

        	public void setStylesheet( String path )
        	{
          
            		if ( _docHandler != null )
                		throw new 
					IllegalStateException( "Cannot set stylesheet after obtaining document handler" );
            
            		try {
                		_stylesheet = _xslReader.read(path);
            		} catch(Exception ex) {
                		throw new IllegalArgumentException(ex.toString());
            		}//try           
        	}//setStyleSheet 

        	public void setStylesheet( InputSource source )
        	{
            		if ( _docHandler != null )
                		throw new IllegalStateException( "Cannot set stylesheet after obtaining document handler" );
                
            		String uri = source.getSystemId();
            		try {
                		URILocationImpl location 
                    			= new URILocationImpl(source.getCharacterStream(), uri);
                		_stylesheet = _xslReader.read(location);
            		} catch(Exception ex) {
                		throw new IllegalArgumentException(ex.toString());
            		}//try
        	}//setStyleSheet(InputSource

        	public void setOutputFormat( OutputFormat format )
        	{
            		if ( _docHandler != null )
                		throw new IllegalStateException( "Cannot set stylesheet after obtaining document handler" );

            		_format = format;
        	}//setOutputFormat

        	public void setOutputMethod( String method )
        	{
            		if ( _docHandler != null )
                		throw new IllegalStateException( "Cannot set stylesheet after obtaining document handler" );

            		if ( _format != null )
                		_format.setMethod( method );
            		else
                		_format = new OutputFormat( method, null, false );
        	}//setOutputMethod


        	public String getCharacterEncoding()
        	{
            		return _response.getCharacterEncoding();
        	}//getCharacterEncoding

        	public ServletOutputStream getOutputStream()
            		throws IOException
        	{
            		return _response.getOutputStream();
        	}//getOutputStream

        	public PrintWriter getWriter()
            		throws IOException
        	{
            		return _response.getWriter();
        	}//getWriter

        	public void setContentLength( int length )
        	{
            		_response.setContentLength( length );
        	}//setContentLength


        	public void setContentType( String type )
        	{
            		_response.setContentType( type );
        	}//setContentType


        	public void setBufferSize( int size )
        	{
            		_response.setBufferSize( size );
        	}//setBufferSize


        	public int getBufferSize()
        	{
            		return _response.getBufferSize();
        	}//getBufferSize

        	public void flushBuffer()
            		throws IOException
        	{
            		_response.flushBuffer();
        	}//flushBuffer

        	public boolean isCommitted()
        	{
            		return _response.isCommitted();
        	}//isCommitted

        	public void reset()
        	{
            		_response.reset();
        	}//reset

        	public void resetBuffer()
        	{
            		_response.resetBuffer();
        	}//resetBuffer
 
        	public void setLocale( Locale locale )
        	{
            		_response.setLocale( locale );
        	}//setLocale

        	public Locale getLocale()
        	{
            		return _response.getLocale();
        	}//getLocale

        	public void addCookie( Cookie cookie )
        	{
            		_response.addCookie( cookie );
        	}//addCookie

        	public boolean containsHeader( String name )
        	{
            		return _response.containsHeader( name );
        	}//containsHeader


        	public String encodeURL( String url )
        	{
            		return _response.encodeURL( url );
        	}//encodeURL


        	public String encodeRedirectURL( String url )
        	{
            		return _response.encodeRedirectURL( url );
        	}//encodeRedirectURL

        	/**
         	* @deprecated
         	*/
        	public String encodeUrl( String url )
        	{
            		return _response.encodeURL( url );
        	}//encodeURL

        	/**
         	* @deprecated
         	*/
        	public String encodeRedirectUrl( String url )
        	{
            		return _response.encodeRedirectURL( url );
        	}//encodeRedirectURL

        	public void sendError( int sc, String msg )
            		throws IOException
        	{
            		_response.sendError( sc, msg );
        	}//sendError

        	public void sendError( int sc )
            		throws IOException
        	{
            		_response.sendError( sc );
        	}//sendError(int sc

        	public void sendRedirect( String location )
            		throws IOException
        	{
            		_response.sendRedirect( location );
        	}//sendRedirect

        	public void setDateHeader( String name, long date )
        	{
            		_response.setDateHeader( name, date );
        	}//setDateHeader

        	public void addDateHeader( String name, long date )
        	{
            		_response.addDateHeader( name, date );
        	}//addDateHeader

        	public void setHeader( String name, String value )
        	{
            		_response.setHeader( name, value );
        	}//setHeader

        	public void addHeader( String name, String value )
        	{
            		_response.addHeader( name, value );
        	}//addHeader


        	public void setIntHeader( String name, int value )
        	{
            		_response.setIntHeader( name, value );
        	}//setIntHeader

        	public void addIntHeader( String name, int value )
        	{
            		_response.addIntHeader( name, value );
        	}//addIntHeader

        	public void setStatus( int sc )
        	{
            		_response.setStatus( sc );
        	}//setStatus

        	public void setStatus( int sc, String msg )
        	{
            		_response.setStatus( sc );
        	}//setStatus(int sc, String msg

	}//XMLServletResponseImpl
}//XMLServlet
