/*
 * Client.java - sample SOAP client.
 *
 * 		Art Rhyno (arhyno@uwindsor.ca), Sept. 2002
 *		(c) Copyright GNU General Public License (GPL)
 *
 * 		Revised:
 *
 */
package soapclient;

import javax.xml.messaging.URLEndpoint;
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPMessage;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamResult;
import soapclient.Fault;
import soapclient.RequestMessage;

public class Client {

	static final String pyservice = "http://localhost:8080/pyserv/pysoap";

	public static void main (String args[]) {

		System.out.println("starting");

		if (1 != args.length) {
			System.err.println("Usage: java " +
				"pytest.Client <name>");
		} else {
			new Client(args[0]);
		}//if

	}//main

	public Client(String name) {

		try {

			URLEndpoint endpoint =
				new URLEndpoint(pyservice);

			SOAPConnectionFactory scf =
				SOAPConnectionFactory.newInstance();

			SOAPConnection connection =
				scf.createConnection();

			RequestMessage reqMsg =
				new RequestMessage();

			reqMsg.setName(name);

			SOAPMessage replySOAP =
				connection.call(reqMsg.getMessage(),
					endpoint);

			if (Fault.hasFault(replySOAP)) {

				Fault fault =
					new Fault(replySOAP);

				System.err.println("Received SOAP Fault");
				System.err.println("Fault code: " +
					fault.getFaultCode());
				System.err.println("Fault string: " +
					fault.getFaultString());
				System.err.println("Fault detail: " +
					fault.getFaultDetail());
			} else {
				ResponseMessage respMsg = new ResponseMessage(replySOAP);
				String responseValue = null;

				try {
					responseValue = respMsg.getValue();
					System.out.println("Response: " + responseValue);
				} catch (ProcessException e) {
					System.err.println("Parsing error");
				}
			}

			connection.close();

		} catch (Exception e) {
			System.err.println(e.toString());
		}//try
	}//Client
}//Client