/*
 * FaultMessage.java - deal with SOAP problems.
 *
 * 		Art Rhyno (arhyno@uwindsor.ca), Sept. 2002
 *		(c) Copyright GNU General Public License (GPL)
 *
 * 		Revised:
 *
 */
package soapclient;

import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPElement;	
import javax.xml.soap.SOAPFault;
import javax.xml.soap.Detail;
import javax.xml.soap.DetailEntry;
import java.util.Iterator;

public class Fault {

	protected SOAPFault soapFault;

	public Fault(SOAPMessage msg)
		throws SOAPException {

		SOAPPart soapPart = msg.getSOAPPart();
		SOAPEnvelope soapEnvelope =
			soapPart.getEnvelope();
		SOAPBody soapBody = soapEnvelope.getBody();
		soapFault = soapBody.getFault();

	}//Fault

	public static boolean hasFault(SOAPMessage msg)
		throws SOAPException
	{
		SOAPPart soapPart = msg.getSOAPPart();
		SOAPEnvelope soapEnvelope =
			soapPart.getEnvelope();
		SOAPBody soapBody = soapEnvelope.getBody();

		return soapBody.hasFault();
	}//hasFault

	public String getFaultCode() {
		return soapFault.getFaultCode();
	}//getFaultCode

	public String getFaultString() {
		return soapFault.getFaultString();
	}//getFaultString

	public String getFaultDetail() {
		String ret = null;
		Detail detail = soapFault.getDetail();

		if (null != detail) {
			Iterator it =
				detail.getDetailEntries();

			if (it.hasNext()) {
				DetailEntry detEntry =
					(DetailEntry) it.next();
				ret = detEntry.getValue();
			}//if it
		}//if

		return ret;
	}//getFaultDetail
	
}//Fault